require 'uu/os/rest'
require 'uu/os/qos/qos_handler'
require 'uu/os/widget/widget_attributes'
require 'uu/os/widget/widget_create'
require 'uu/os/widget/widget_get_widget_list_item'
require 'uu/os/widget/widget_get_widget_list'
require 'uu/os/widget/widget_set_attributes'
require 'uu/os/widget/widget_get_export_data'

# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # Module widget.
    module Widget
      extend self

      # Service path
      PATH = 'uu/os/Widget'
      # Creates a widget in specified location. At the very least meta artifact must be specified in WidgetCreate object. Competent role for the new widget will be selected as the most suitable according to specified package and meta artifact (executive/authorized role with connected interface),
      # or can be also specified in WidgetCreate Object.
      #
      # @param [String, UU::OS::UESURI] location_uri URI of an uuApp Package where the widget will be created.
      # @param [Object] widget Object containing the attributes of the widget.
      # @option widget [String] :name Name of the new widget. When nil is set, default name from meta artifact is used if set, otherwise the create command fails.
      # @option widget [String] :code Code of the new widget. When nil is set, the code is generated automatically using the system sequence. The code must be unique in the given territory, otherwise the create command fails.
      # @option widget [String] :description Description of the new widget. When nil is set, default description from meta artifact is used if set, otherwise no description is set on the new widget.
      # @option widget [String] :security_level Security level. When nil is set, security level is taken from the meta artifact default security level. If the meta artifact template security level is higher than the user's security clearance, the create command fails.
      # @option widget [String, UU::OS::UESURI] :icon_uri UESURI of the widget's icon. When nil is set, the role uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option widget [String, UU::OS::UESURI] :competent_role_uri UESURI of the competent role. When nil is set, the executive role from authorization is used. If no role is authorized to create the widget into specified container, the create command fails. If the selected role doesn't have connected the role interface with the selected meta artifact, the create command fails. If the selected role doesn't belong to the same organizational unit as a new widget or if it doesn't belong to one of superior organizational units, the create command fails.
      # @option widget [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the meta artifact. Only a meta artifact with the template class of widget (not its descendants) can be used. If the rights does not permit to create a new widget from the meta artifact in the specified location, the create command fails.
      # @option widget [String] :controller_url_path Path at which the widget's controller is deployed. The value can be a relative or absolute URL (absolute URL has to be in format https://&lt;something&gt;.plus4u.net/...). If an absolute URL is set, it is used as entered, without adding any postfix (e.g. /render which is added for relative URL). If it is not set, the widget cannot be used in the sheet content.
      # @option widget [Boolean] :trusted Describes whether the widget will by inserted into the HTML directly or in the iframe. The default value is false.
      # @option widget [Boolean] :editor_available Describes whether the widget will be in the list of available widgets in editor. The default value is true.
      # @option widget [String, UU::OS::Widget::PlaceholderImage] :placeholder_image Placeholder image of the widget in editor. The default value is GENERAL.
      # @option widget [Number] :width Default width of the widget in content. It must be set together with height. If the attributes width and height or placeholder_width and placeholder_height are not set, the widget cannot be used in the sheet content.
      # @option widget [Number] :height Default height of the widget in content. It must be set together with width. If the attributes width and height or placeholder_width and placeholder_height are not set, the widget cannot be used in the sheet content.
      # @option widget [Number] :placeholder_width  Width of the widget placeholder in editor. It is defined for widgets for which it is not possible to set width and height. It must be set together with placeholder_height. If the attributes width and height or placeholder_width and placeholder_height are not set, the widget cannot be used in the sheet content.
      # @option widget [Number] :placeholder_height  Height of the widget placeholder in editor. It is defined for widgets for which it is not possible to set width and height. It must be set together with placeholder_width. If the attributes width and height or placeholder_width and placeholder_height are not set, the widget cannot be used in the sheet content.
      #
      # @return [UU::OS::UESURI]  Returns the URI of the newly created widget.
      #
      # @example
      #  # Create a new widget.
      #  UU::OS::Widget.create('ues:TERRITORY:PACKAGE',
      #    :meta_artifact_uri => 'ues:TERRITORY:UU.OS/RUNTIME/WIDGET',
      #    :name => 'New Widget',
      #    :code => 'NEW_WIDGET',
      #    :description => 'My new widget',
      #    :controller_url_path => '',
      #    :trusted => false,
      #    :editor_available => true,
      #    :placeholder_image => UU::OS::Widget::PlaceholderImage::GENERAL,
      #    :width => 200,
      #    :height => 200
      #  )
      def create(location_uri, widget = nil)
        svc = UU::OS::REST::RemoteClient.new(Widget)
        payload = UU::OS::Widget::WidgetCreate.new(widget).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Deletes the specified widget from the system. The widget is deleted even if it contains a locked sheet or attachment. The widget cannot be deleted when:
      #the widget's workflow is not created
      #a property contains a reference to this widget
      #a time sheet exists with a reference to this widget
      #
      # @param [String, UU::OS::UESURI] widget_uri URI of the widget which is about to be deleted.
      #
      # @example
      #   # Delete widget with specified URI.
      #   UU::OS::Widget.delete('ues:TERRITORY:WIDGET')
      def delete(widget_uri)
        svc = UU::OS::REST::RemoteClient.new(Widget)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', widget_uri)
        end
      end

      # Returns attributes of the specified widget.
      #
      # @param [String, UU::OS::UESURI] widget_uri URI of the required widget.
      #
      # @return [WidgetAttributes] Returns object containing attributes of selected widget.
      #
      # @example
      #   # Get widget attributes.
      #   attributes = UU::OS::Widget.get_attributes('ues:TERRITORY:WIDGET')
      def get_attributes(widget_uri)
        svc = UU::OS::REST::RemoteClient.new(Widget)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', widget_uri)
          return Widget::WidgetAttributes.new(res)
        end
      end

      #Sets attributes of widget. Command cannot modify references to objects like location, competent role and so on.
      #Changing the code of the widget will cause malfunction of widget instances. #
      # @param [String, UU::OS::UESURI] widget_uri URI of the widget which attributes will be changed.
      # @param [Object] widget Object containing new attributes of the widget.
      # @option widget [String] :name The new name of the widget. When nil is set, the name is not modified.
      # @option widget [String] :code The new code of the widget. When nil is set, the code is not modified.
      # @option widget [String] :description The new description of the widget. When nil is set, the description is not modified.
      # @option widget [String] :security_level Security level of the widget. When nil is set, the security level is not modified.
      # @option widget [String, UU::OS::UESURI] :icon_uri UESURI of the widget icon. When nil is set, the icon is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:") is set, the widget icon is removed and the widget uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option widget [String] :controller_url_path Path at which the widget's controller is deployed. The value can be a relative or absolute URL (absolute URL has to be in format https://&lt;something&gt;.plus4u.net/...). If an absolute URL is set, it is used as entered, without adding any postfix (e.g. /render which is added for relative URL). If it is not set, the widget cannot be used in the sheet content.
      # @option widget [Boolean] :trusted Describes whether the widget will by inserted into the HTML directly or in the iframe. The default value is false.
      # @option widget [Boolean] :editor_available Describes whether the widget will be in the list of available widgets in editor. The default value is true.
      # @option widget [String, UU::OS::Widget::PlaceholderImage] :placeholder_image Placeholder image of the widget in editor. The default value is GENERAL.
      # @option widget [Number] :width Default width of the widget in content. It must be set together with height. If the attributes width and height or placeholder_width and placeholder_height are not set, the widget cannot be used in the sheet content.
      # @option widget [Number] :height Default height of the widget in content. It must be set together with width. If the attributes width and height or placeholder_width and placeholder_height are not set, the widget cannot be used in the sheet content.
      # @option widget [Number] :placeholder_width  Width of the widget placeholder in editor. It is defined for widgets for which it is not possible to set width and height. It must be set together with placeholder_height. If the attributes width and height or placeholder_width and placeholder_height are not set, the widget cannot be used in the sheet content.
      # @option widget [Number] :placeholder_height  Height of the widget placeholder in editor. It is defined for widgets for which it is not possible to set width and height. It must be set together with placeholder_width. If the attributes width and height or placeholder_width and placeholder_height are not set, the widget cannot be used in the sheet content.
      #
      # @return [UU::OS::UESURI] Returns URI of the changed widget.
      #
      # @example
      #  # Set widget attributes.
      #  UU::OS::Widget.set_attributes('ues:TERRITORY:WIDGET',
      #    :name => 'New Widget Name',
      #    :code => 'NEW_WIDGET_CODE',
      #    :description => 'My new widget',
      #    :editor_available => false,
      #    :width => 300,
      #    :height => 300,
      #  )
      def set_attributes(widget_uri, widget = nil)
        svc = UU::OS::REST::RemoteClient.new(Widget)
        payload = UU::OS::Widget::WidgetSetAttributes.new(widget).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', widget_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns the list of widgets whose instances can be inserted into the content of selected sheet or into the content of a sheet on a given Artifact. Returned list does not contain any objects filtered out by executed UESQuery.
      # @param [String, UU::OS::UESURI] entity_uri URI of the sheet or Artifact, which widget list will be returned.
      # @param [Object] criteria Specific options.
      # @option criteria [String] :query The <b>UES query</b> string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there. It is possible to filter the result list by name and code of widget. When nil is set, no filtering is applied and entries are ordered by name of referenced artifact (and code and type as the second and the third criteria).
      #
      # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      #
      # @return [UU::OS::REST::ResultList<Widget::WidgetGetWidgetListItem, Widget::WidgetGetWidgetList>]  Returns list of Objects representing widgets which can be used in the selected sheet content or in the content of a sheet on a given Artifact.
      #
      # @example
      #  UU::OS::Widget.get_widget_list('ues:TERRITORY:ARTIFACT:SHEET', criteria = nil)
      def get_widget_list(entity_uri, criteria=nil)
        svc = UU::OS::REST::RemoteClient.new(Widget)
        dto = Widget::WidgetGetWidgetList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getWidgetList', entity_uri)

          return UU::OS::REST::ResultList.new(Widget::WidgetGetWidgetList, Widget::WidgetGetWidgetListItem, res)
        end
      end

      #Returns the list of widgets, which are registered in the selected territory or uuApp meta model. Returned list does not contain widgets located in the system territory. Returned list does not contain any objects filtered out by executed UESQuery.    #
      # @param [String, UU::OS::UESURI] container_uri  URI of the territory or uuApp meta model whose registered widget will be returned.
      # @param [Object] criteria Specific options.
      # @option criteria [String] :query The <b>UES query</b> string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there. It is possible to filter the result list by name and code of widget. When nil is set, no filtering is applied and entries are ordered by name of referenced artifact (and code and type as the second and the third criteria).
      #
      # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      #
      # @return [UU::OS::REST::ResultList<Widget::WidgetGetWidgetListItem, Widget::WidgetGetWidgetList>] Returns list of Objects representing widgets which are registered in the selected territory/uuApp meta model.
      #
      # @example
      #   UU::OS::Widget.get_registered_widget_list('ues:TERRITORY:TERRITORY', criteria = nil)
      def get_registered_widget_list(container_uri, criteria=nil)
        svc = UU::OS::REST::RemoteClient.new(Widget)
        dto = Widget::WidgetGetWidgetList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getRegisteredWidgetList', container_uri)
          return UU::OS::REST::ResultList.new(Widget::WidgetGetWidgetList, Widget::WidgetGetWidgetListItem, res)
        end
      end

      #Exports a widget to the XML file, which is saved to the export storage. The export is an asynchronous process.
      #It is possible to wait for the end of the process. More information in {UU::OS::Env::Process} and in {UU::OS::REST::Future}.
      #
      # @param [String, UU::OS::UESURI] widget_uri URI of the widget to export.
      # @return [UU::OS::UESURI] Returns URI of the export process.
      #
      # @example
      #   # Start exporting a widget.
      #   process_uri = UU::OS::Widget.export('ues:TERRITORY:WIDGET')
      #
      #   # Wait for the end of the process.
      #   export_uri = UU::OS::REST::Future.new(process_uri).get(nil, 600)
      #
      #   # Get exported XML file.
      #   exported_data = UU::OS::Widget.get_export_data(export_uri, :requested_mime_type => 'application/xml')

      def export(widget_uri)
        svc = UU::OS::REST::RemoteClient.new(Widget)
        payload = UU::OS::Widget::WidgetExport.new(nil).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('export', widget_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns a XML file generated by the export command.{@link Widget#export}.
      #
      # @param [String, UU::OS::UESURI] widget_uri URI of the widget to export.
      # @param [UU::OS::Widget::WidgetGetExportData] widget Specific options.
      # @option widget [String] :requested_mime_type MIME type of the returned value. All possible MIME types can be found as constants in {@link UU::OS::Artifact::ExportMimeType} class. ZIP is returned by default.
      #
      # @return [UU::OS::REST::BinaryValue] Data file of an exported widget (XML or ZIP containing XML).
      #
      # @example
      #   # Start exporting a widget.
      #   process_uri = UU::OS::Widget.export('ues:TERRITORY:WIDGET')
      #
      #   # Wait for the end of the process.
      #   export_uri = UU::OS::REST::Future.new(process_uri).get(nil, 600)
      #
      #   # Get exported XML file.
      #   exported_data = UU::OS::Widget.get_export_data(export_uri, :requested_mime_type => 'application/xml')
      def get_export_data(widget_uri,widget = nil)
        svc = UU::OS::REST::RemoteClient.new(Widget)
        dto = UU::OS::Widget::WidgetGetExportData.new(widget)
        if dto.requested_mime_type
          svc.add_parameter('requestedMimeType', dto.requested_mime_type)
        end
        UU::OS::QoS::QoSHandler.auto_retry do
          return svc.get_binary('getExportData', widget_uri)
        end
      end

    end

  end
end
